//
//  ISTappxCustomBanner.m
//
//  Created by Rubén on 28/01/25.
//

#import "ISTappxCustomBanner.h"

@interface ISTappxCustomBanner ()

@property (strong, nonatomic) TappxBannerViewController* bannerAd;
@property (nonatomic, strong) UIView *adView;
@property (strong, nonatomic) id<ISBannerAdDelegate> adDelegate;

@end

@implementation ISTappxCustomBanner

static UIViewController* rootVC;

+ (UIViewController *) _getRootVC {
    return rootVC;
}

+ (void) _setRootVC:(UIViewController *) vc {
    rootVC = vc;
}

typedef NS_ENUM (NSInteger) {
    IS_NO_FILL = 0,
    IS_SERVER_ERROR,
    IS_DEVELOPER_ERROR,
    IS_VIEW_INCONSISTENCY_ERROR,
    IS_CANCELLED,
    IS_NO_CONNECTION
} ISTappxError;

- (void)loadAdWithAdData:(nonnull ISAdData *)adData
     viewController:(UIViewController *)viewController
          size:(ISBannerSize *)size
        delegate:(nonnull id<ISBannerAdDelegate>)delegate{
    
    _adDelegate = delegate;
    if (adData != nil){
        NSString *tappxKey = [adData.configuration valueForKey:@"tappxkey"];
        NSString *isTest = [adData.configuration valueForKey:@"test"];
        if (isTest != nil && [isTest isEqualToString:@"1"]){
            [TappxFramework addTappxKey:tappxKey testMode:YES];
        }
        else {
            [TappxFramework addTappxKey:tappxKey fromNonNative:@"ironsource"];
        }
    
    } else {
        [self descriptionError:IS_NO_FILL andDescription: @"Missing AdUnit"];
        return;
    }
    
    TappxBannerSize tappxsize = TappxBannerSmartBanner;
    CGRect sizeFrame = CGRectMake(0, 0, (([[UIDevice currentDevice] userInterfaceIdiom]==UIUserInterfaceIdiomPhone) ? 320 : 728 ), (([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) ? 50 : 90));
    if (size.sizeDescription == kSizeBanner){
        tappxsize = TappxBannerSize320x50;
        sizeFrame = CGRectMake(0, 0, 320, 50);
    }else if (size.sizeDescription == kSizeLarge){
        tappxsize = TappxBannerSize728x90;
        sizeFrame = CGRectMake(0, 0, 728, 90);
    }else if (size.sizeDescription == kSizeRectangle){
        tappxsize = TappxBannerSize300x250;
        sizeFrame = CGRectMake(0, 0, 300, 250);
    }
    
    self.adView = [[UIView alloc] initWithFrame:sizeFrame];
    self.bannerAd = [[TappxBannerViewController alloc] initWithDelegate:self andSize:tappxsize andView:self.adView];
    [self.bannerAd load];

}

- (void) destroyAdWithAdData:(nonnull ISAdData *)adData {
    if(self.bannerAd != nil){
        [self.bannerAd removeBanner];
        self.bannerAd = nil;
    }
    
    if(self.adView != nil) {
        self.adView = nil;
    }
}

- (void)dealloc {
    if(self.bannerAd != nil){
        [self.bannerAd removeBanner];
        self.bannerAd = nil;
    }
    
    if(self.adView != nil) {
        self.adView = nil;
    }
}
- (void)present:(nonnull UIViewController *)viewController withCompletions:(void (^ _Nonnull)(void))completion {
    [[ISTappxCustomBanner _getRootVC] presentViewController:viewController animated:false completion:completion];
}
- (nonnull UIViewController *)presentViewController {
    return [ISTappxCustomBanner _getRootVC];
}

//TAPPXBannerDelegate

- (void) tappxBannerViewControllerDidFinishLoad:(TappxBannerViewController*) vc {
    [_adDelegate adDidLoadWithView:self.adView];
    [_adDelegate adDidOpen];
}

- (void) tappxBannerViewControllerDidPress:(TappxBannerViewController*) vc {
    [_adDelegate adDidClick];
}

- (void) tappxBannerViewControllerDidFail:(TappxBannerViewController*) vc withError:(TappxErrorAd*) error {
    [self descriptionError:error.code andDescription:error.localizedDescription];
    
    if(self.bannerAd != nil){
        [self.bannerAd removeBanner];
        self.bannerAd = nil;
    }
    if (self.adView != nil)
        self.adView = nil;
}
- (void) tappxBannerViewControllerDidClose:(TappxBannerViewController*) vc {
    [_adDelegate adDidDismissScreen];
}

- (void)descriptionError:(NSInteger)errorCode andDescription:(nullable NSString *)description
{
    NSString *errorString = @"IS Error: ";
    NSInteger tappxErrorCode = 0;
    ISAdapterErrorType errorType;
    
    switch (errorCode) {
        case IS_NO_FILL:
            errorString = [errorString stringByAppendingString:@"IS_NO_FILL"];
            errorType = ISAdapterErrorTypeNoFill;
            tappxErrorCode = IS_NO_FILL;
            break;
            
        case IS_SERVER_ERROR:
            errorString = [errorString stringByAppendingString:@"IS_SERVER_ERROR"];
            errorType = ISAdapterErrorInternal;
            tappxErrorCode = IS_SERVER_ERROR;
            break;
            
        case IS_DEVELOPER_ERROR:
            errorString = [errorString stringByAppendingString:@"IS_DEVELOPER_ERROR"];
            errorType = ISAdapterErrorMissingParams;
            tappxErrorCode = IS_DEVELOPER_ERROR;
            break;
            
        case IS_VIEW_INCONSISTENCY_ERROR:
            errorString = [errorString stringByAppendingString:@"IS_VIEW_INCONSISTENCY_ERROR"];
            errorType = ISAdapterErrorTypeInternal;
            tappxErrorCode = IS_VIEW_INCONSISTENCY_ERROR;
            break;
            
        case IS_CANCELLED:
            errorString = [errorString stringByAppendingString:@"IS_CANCELLED"];
            errorType = ISAdapterErrorTypeInternal;
            tappxErrorCode = IS_CANCELLED;
            break;
            
        case IS_NO_CONNECTION:
            errorString = [errorString stringByAppendingString:@"NO_CONNECTION"];
            errorType = ISAdapterErrorTypeInternal;
            tappxErrorCode = IS_NO_CONNECTION;
            break;
        
        default:
            errorString = [errorString stringByAppendingString:@"IS_SERVER_ERROR"];
            errorType = ISAdapterErrorTypeInternal;
            tappxErrorCode = IS_SERVER_ERROR;
            break;
    }
    
    if(description) {
        errorString = [[errorString stringByAppendingString:@" "] stringByAppendingString:description];
    }
    
    if (tappxErrorCode == IS_VIEW_INCONSISTENCY_ERROR){
        //[_adDelegate adDidFailToLoadWithErrorType:errorType errorCode:tappxErrorCode errorMessage:errorString];
        [_adDelegate adDidFailToShowWithErrorCode:tappxErrorCode errorMessage:errorString];
        return;
    }
    
    [_adDelegate adDidFailToLoadWithErrorType:errorType errorCode:tappxErrorCode errorMessage:errorString];
}


@end

